﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Threading;
using System.Windows.Threading;
using MindFusion.RealTimeCharting.Wpf;
using System.Media;
using System.ComponentModel;

namespace RealTimeChart_Music
{
	/// <summary>
	/// Interaction logic for Window1.xaml
	/// </summary>
	public partial class Window1 : Window, INotifyPropertyChanged
	{
		private WaveReader reader;
		private Series audioSerie;
		private Series position;
		private Point startPos;
		private Point endPos;
		private Axis yAxis;
		private Axis yAxisRight;
		private long bytes;
		private int seconds;
		private int index;
		private string filename;
		private SoundPlayer player;
		private DateTime startTime;
		private System.Windows.Threading.DispatcherTimer dispatcherTimer;
		private System.Windows.Threading.DispatcherTimer lineTimer;
		public ulong loadedPoints;

		public Window1()
		{
			InitializeComponent();
			init();
			musicGraphic.YAxisCollection.Add(yAxis);
			musicGraphic.Y2AxisCollection.Add(yAxisRight);
			lineTimer = new System.Windows.Threading.DispatcherTimer(DispatcherPriority.Send);
		}

		private void init()
		{
			seconds = 0;
			bytes = 44;
			index = 0;
			yAxis = new Axis();
			yAxis.Origin = -1;
			yAxis.Length = 2;
			yAxis.Interval = 0.2;
			yAxisRight = new Axis();
			yAxisRight.Origin = -1;
			yAxisRight.Length = 2;
			yAxisRight.Interval = 0.2;
			yAxisRight.LabelHorizontalAlignment = HorizontalAlignment.Right;
			audioSerie = new Series(yAxis);
			startPos = new Point(0, -1);
			endPos = new Point(0, 1);
			filename = "";
			loadedPoints = 0;
			musicGraphic.XAxis = new Axis();
			musicGraphic.XAxis.Title = "Time in seconds";
		}

		public ulong LoadedPoints
		{
			get { return loadedPoints; }
			set 
			{
				loadedPoints = value;
				OnPropertyChanged("LoadedPoints");
			}
		}

		private void OnLoaded(object sender, RoutedEventArgs e)
		{
			musicGraphic.OnApplyTemplate();
			Axis a = musicGraphic.XAxis;
			musicGraphic.IndicatorLine.Y2 = musicGraphic.PlotArea.ActualHeight - (a.LabelFontSize + a.TickLength + 2 * a.LabelOffset + (a.Title!=null && a.Title!=""?(a.TitleFontSize + a.TitleOffset):0));
			loadFile("thats_it.wav"); /*downloaded from http://www.nichecom.com/songs/*/
		}

		private void AddGraphics(object sender, EventArgs e)
		{
			int onePointBytes = reader.Format.BitsPerSample/8 * reader.Format.Channels;
			int pointsInMB = 1024 * 1024 / onePointBytes;
			int bytesPerSecond = (int)reader.Data.Size / (int)reader.Duration;
			if (seconds < reader.Duration)
			{
				using (var viewAccessor = reader.File.CreateViewAccessor(bytes, bytesPerSecond))
				{
					int old = index;
					for (int i = 0; i < reader.Format.SamplesPerSecond; i += 3 * (int)reader.Duration * reader.Format.Channels)
					{
						if (reader.Format.BitsPerSample == 8)
						{
							double pointY = (((double)viewAccessor.ReadByte(i)) - 128.0)/128.0;
							double pointX = (double)seconds + (double)i / reader.Format.SamplesPerSecond;
							Point toAdd = new Point(pointX, pointY);
							audioSerie.Data.Add(toAdd);
						}
						if (reader.Format.BitsPerSample == 16)
						{
							double pointY = (double)viewAccessor.ReadInt16(i)/42000.0;
							double pointX = (double)seconds + (double)i / reader.Format.SamplesPerSecond;
							Point toAdd = new Point(pointX, pointY);
							audioSerie.Data.Add(toAdd);
						}
						if (reader.Format.BitsPerSample == 32)
						{
							double pointY = (double)viewAccessor.ReadSingle(i);
							double pointX = (double)seconds + (double)i / reader.Format.SamplesPerSecond;
							Point toAdd = new Point(pointX, pointY);
							audioSerie.Data.Add(toAdd);
						}
						LoadedPoints++;
						index++;
					}
					musicGraphic.Commit(old);
					seconds++;
				}
				bytes += bytesPerSecond;
			}
			else
			{
				reader.File.Dispose();
				player = new SoundPlayer(filename);
				dispatcherTimer.Stop();
				Play.IsEnabled = true;	
			}
		}

		private void DrawPosition(object sender, EventArgs e)
		{
			TimeSpan currentTime = DateTime.Now - startTime;
			double offset = (double)musicGraphic.LineCanvas.ActualWidth * (currentTime.TotalSeconds / ((double)reader.Data.Size/(double)reader.Format.AverageBytesPerSecond));
			if (musicGraphic.IndicatorLine.X1 < musicGraphic.LineCanvas.ActualWidth)
			{
				musicGraphic.IndicatorLine.X1 = offset;
				musicGraphic.IndicatorLine.X2 = offset;
			}
			if (musicGraphic.IndicatorLine.X1 > musicGraphic.LineCanvas.ActualWidth)
			{
				musicGraphic.IndicatorLine.X1 = musicGraphic.LineCanvas.ActualWidth;
				musicGraphic.IndicatorLine.X2 = musicGraphic.LineCanvas.ActualWidth;
				lineTimer.Stop();
				Stop.IsEnabled = false;
				Play.IsEnabled = true;
			}
		}

		private void Play_Click(object sender, RoutedEventArgs e)
		{
			if (filename != null && filename != "")
			{
				musicGraphic.IndicatorLine.X1 = 0;
				musicGraphic.IndicatorLine.X2 = 0;
				player.Play();
				startTime = DateTime.Now;
				lineTimer = new System.Windows.Threading.DispatcherTimer(DispatcherPriority.Send);
				lineTimer.Tick += new EventHandler(DrawPosition);
				lineTimer.Interval = new TimeSpan(0, 0, 0, 0, 100);
				audioSerie.Stroke = new SolidColorBrush(Colors.Red);
				lineTimer.Start();
				Play.IsEnabled = false;
				Stop.IsEnabled = true;
			}
		}

		private void Stop_Click(object sender, RoutedEventArgs e)
		{
			player.Stop();
			lineTimer.Stop();
			Stop.IsEnabled = false;
			Play.IsEnabled = true;
		}

		private void Browse_Click(object sender, RoutedEventArgs e)
		{
			init();
			//musicGraphic.ClearCache();
			Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();

			// Set filter for file extension and default file extension
			dlg.DefaultExt = ".wav";
			dlg.Filter = "Wave (.wav)|*.wav";

			// Display OpenFileDialog by calling ShowDialog method
			Nullable<bool> result = dlg.ShowDialog();

			// Get the selected file name and display in a TextBox
			if (result == true)
			{
				// Open document
				//musicGraphic.FastScrollMode = true;
				filename = dlg.FileName;
				tb2.Text = "File: "+filename;

				loadFile(filename);
			}
		}

		private void loadFile(string filename)
		{
			this.filename = filename;
			reader = new WaveReader(filename);

			musicGraphic.XAxis.Interval = reader.Duration / 10;
			musicGraphic.XAxis.Length = reader.Duration;

			if (musicGraphic.SeriesCollection.Count == 0)
				musicGraphic.SeriesCollection.Add(audioSerie);
			else
			{
				audioSerie.Data.Clear();
				musicGraphic.SeriesCollection[0] = audioSerie;
			}

			musicGraphic.Commit();
			dispatcherTimer = new System.Windows.Threading.DispatcherTimer(DispatcherPriority.Send);
			dispatcherTimer.Tick += new EventHandler(AddGraphics);
			dispatcherTimer.Interval = TimeSpan.FromMilliseconds(10);
			dispatcherTimer.Start();
		}

		private void OnSizeChanged(object sender, SizeChangedEventArgs e)
		{
			Axis a = musicGraphic.XAxis;
			musicGraphic.IndicatorLine.Y2 = musicGraphic.PlotArea.ActualHeight - (a.LabelFontSize + a.TickLength + 2 * a.LabelOffset + (a.Title != null && a.Title != "" ? (a.TitleFontSize + a.TitleOffset) : 0));
		}
		public event PropertyChangedEventHandler PropertyChanged;

		//[NotifyPropertyChangedInvocator]
		protected virtual void OnPropertyChanged(string propertyName)
		{
			PropertyChangedEventHandler handler = PropertyChanged;
			if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
		}

	}
}
